  /*
  Advanced_SPI.ino
  Brian R Taylor
  brian.taylor@bolderflight.com
  Copyright (c) 2017 Bolder Flight Systems
  */
  #include <SPI.h>
  #include "MPU9250.h"
  #include <Adafruit_BMP280.h>

  #define BMP_CS            5
  #define IMU_CS           12
  #define  START_Pin        3
  #define  PUSK_Pin        35             // назначение вывода PUSK
  #define  POZ_Pin         10             // назначение вывода START

  bool flagknt,flagknp,flagkn = 0;      // флаг текущего, прошлого состояния и нажатия кнопки Enter
  bool flpuskt,flpuskp,flpusk = 0;      // флаг текущего, прошлого состояния и нажатия кнопки Pusk
  bool flstart,flstarp,flstar = 0;      // флаг текущего, прошлого состояния и нажатия кнопки Start
  bool zapisy = 0;                      // флаг процесса записи на карту памяти
  float  ax, ay, az, hxb, hyb, hzb, hxs, hys, hzs, mx, my, mz = 0;

  MPU9250 IMU(SPI,IMU_CS);
  Adafruit_BMP280 bmp(BMP_CS); 

  float Direction;
  const float Pi = 3.14159;

  void setup() {

  pinMode(START_Pin, INPUT);            // назначение вывода Enter_Pin входом 
  pinMode(PUSK_Pin, INPUT_PULLUP);      // назначение вывода PUSK_Pin входом
  pinMode(POZ_Pin, INPUT_PULLUP);     // назначение вывода START_Pin входом

  // serial to display data
  Serial.begin(115200);
  while(!Serial) {}
  delay(5000);

  // start communication with IMU 
  IMU.begin();
  // setting the accelerometer full scale range to +/-2G 
  IMU.setAccelRange(MPU9250::ACCEL_RANGE_2G);
  // setting the gyroscope full scale range to +/-500 deg/s
  // IMU.setGyroRange(MPU9250::GYRO_RANGE_500DPS);
  // setting DLPF bandwidth to 20 Hz
  IMU.setDlpfBandwidth(MPU9250::DLPF_BANDWIDTH_20HZ);
  // setting SRD to 19 for a 50 Hz update rate
  IMU.setSrd(19);

  IMU.setMagCalX(28, 1.12);
  IMU.setMagCalY(20, 1);
  IMU.setMagCalZ(-38, 0.9);
 
  bmp.begin(BMP280_ADDRESS_ALT, BMP280_CHIPID);

  /* Default settings from datasheet. */
  bmp.setSampling(Adafruit_BMP280::MODE_NORMAL,     /* Operating Mode. */
                  Adafruit_BMP280::SAMPLING_X2,     /* Temp. oversampling */
                  Adafruit_BMP280::SAMPLING_X16,    /* Pressure oversampling */
                  Adafruit_BMP280::FILTER_X16,      /* Filtering. */
                  Adafruit_BMP280::STANDBY_MS_500); /* Standby time. */
   }

  void CalMag() {

  IMU.setMagCalX(0, 1);
  IMU.setMagCalY(0, 1);
  IMU.setMagCalZ(0, 1);
   Serial.println("\n-------------[ SLOWLY AND CONTINUOUSLY MAKE A FIGURE 8 MOTION DURING CALIBRATION ]-------------");

  // Countdown timer
  for (int i = 5; i > 0; i--)
  {
  Serial.print(i);
  Serial.println("...");
  delay(1000);
  }

  if (IMU.calibrateMag() == true)
  {
  // Gather magnetometer bias/scale factors
  hxb = IMU.getMagBiasX_uT();
  hyb = IMU.getMagBiasY_uT();
  hzb = IMU.getMagBiasZ_uT();
  hxs = IMU.getMagScaleFactorX();
  hys = IMU.getMagScaleFactorY();
  hzs = IMU.getMagScaleFactorZ();

  // Display bias/scale factors
  Serial.println("\nCALIBRATION COMPLETE\n");
  Serial.println("---- MAG BIAS VALUES ----\n");
  Serial.print("X_Bias : "); Serial.print(hxb); Serial.print(',');
  Serial.print(" Y_Bias : "); Serial.print(hyb); Serial.print(',');
  Serial.print(" Z_Bias : "); Serial.println(hzb);

  Serial.println("\n---- MAG SCALE VALUES ----\n");
  Serial.print("X_Scale : "); Serial.print(hxs); Serial.print(',');
  Serial.print("Y_Scale : "); Serial.print(hys); Serial.print(',');
  Serial.print("Z_Scale : "); Serial.println(hzs);

  // Set biased/scale factors
  IMU.setMagCalX(hxb, hxs);
  IMU.setMagCalY(hyb, hys);
  IMU.setMagCalZ(hzb, hzs);
  Serial.println("\n---- MAGNOMETER BIAS/SCALE FACTOR SET TO ----\n");

  Serial.print("Mag X Bias : "); Serial.print(hxb); Serial.print("\tMag X Scale Factor : "); Serial.println(hxs);
  Serial.print("Mag Y Bias : "); Serial.print(hyb); Serial.print("\tMag Y Scale Factor : "); Serial.println(hys);
  Serial.print("Mag Z Bias : "); Serial.print(hzb); Serial.print("\tMag Z Scale Factor : "); Serial.println(hzs);

  }

  //---------------------------------------------------- [ CALIBRATION STUFF DONE ] ----------------------------------------------------//
  Serial.println("\nPress [ ENTER ] to display IMU data...");

  // Calibration won't start until user inputs 'ENTER' in the serial monitor
  while (true)
  {
  if (digitalRead(START_Pin) == 0) {		          // Если есть нажатие кнопки Enter
  break;
  }
  Serial.print(".");
  delay(1000);
  }
  // Uncomment if you're exporting the IMU data to something like Excel
  //Serial.println("ax,ay,az,gx,gy,gz,mx,my,mz");
  }

  void QryPin () {
  flagknt =  digitalRead(POZ_Pin);
  if (flagknt == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (flagknp == 1 ) {                         // Если 1, первое чтение, 
  flagkn = 1; }                                // устанавливаем флаг калибровки
  } 
  flagknp = flagknt;                           // записываем текущее состояние кнопки в предыдущее

  flpuskt =  digitalRead(PUSK_Pin);
  if (flpuskt == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (flpuskp == 1 ) {                         // Если 1, первое чтение, 
  flpusk = 1; }                                // устанавливаем флаг калибровки
  } 
  flpuskp = flpuskt;                           // записываем текущее состояние кнопки в предыдущее

  flstart =  digitalRead(START_Pin);
  if (flstart == 0 ) {                         // Если 0, смотрим, первое чтение, или нет
  if (flstarp == 1 ) {                         // Если 1, первое чтение, 
  flstar = 1; }                                // устанавливаем флаг калибровки
  } 
  flstarp = flstart;                           // записываем текущее состояние кнопки в предыдущее
  }

  void loop() {
  // read the sensor
  IMU.readSensor();

  // display the data
    Serial.print(F("AccelX = "));
    Serial.print(IMU.getAccelX_mss());
    Serial.println(" m/ss");

    Serial.print(F("AccelY = "));
    Serial.print(IMU.getAccelY_mss());
    Serial.println(" m/ss");

    Serial.print(F("AccelZ = "));
    Serial.print(IMU.getAccelZ_mss());
    Serial.println(" m/ss");
    Serial.println();
    /*
    Serial.print(F("GyroX = "));
    Serial.print(IMU.getGyroX_rads());
    Serial.println(" * rad/s");

    Serial.print(F("GyroY = "));
    Serial.print(IMU.getGyroY_rads());
    Serial.println(" rad/s");

    Serial.print(F("GyroZ = "));
    Serial.print(IMU.getGyroZ_rads());
    Serial.println(" rad/s");
    Serial.println();
    */
  
    Direction = atan2(IMU.getMagX_uT(),IMU.getMagY_uT())*180/Pi;

    Serial.print(F("MagX = "));
    Serial.print(IMU.getMagX_uT());
    Serial.println(" uT");

    Serial.print(F("MagY = "));
    Serial.print(IMU.getMagY_uT());
    Serial.println(" uT");

    Serial.print(F("MagZ = "));
    Serial.print(IMU.getMagZ_uT());
    Serial.println(" uT");

    Serial.print(F("Direction = "));
    Serial.print(Direction);
    Serial.println(" grad");
    Serial.println();

    Serial.print(F("Temperature = "));
    Serial.print(bmp.readTemperature());
    Serial.println(" *C");

    Serial.print(F("Pressure = "));
    Serial.print(bmp.readPressure());
    Serial.println(" Pa");

    Serial.print(F("Approx altitude = "));
    Serial.print(bmp.readAltitude(1013.25)); 
    Serial.println(" m");
    Serial.println();

  QryPin ();
  if (flstar == 1 ) {                           // Если 1, калибровка
  CalMag();
  flstar = 0;
  }

  delay(1000);
  }
